/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright 2020 Mellanox Technologies, Ltd
 */

#ifndef RTE_PMD_PRIVATE_MLX5_H_
#define RTE_PMD_PRIVATE_MLX5_H_

/**
 * @file
 * MLX5 public header.
 *
 * This interface provides the ability to support private PMD
 * dynamic flags.
 */

#ifdef __cplusplus
extern "C" {
#endif

#define RTE_PMD_MLX5_FINE_GRANULARITY_INLINE "mlx5_fine_granularity_inline"

/**
 * Returns the dynamic flags name, that are supported.
 *
 * @param[out] names
 *   Array that is used to return the supported dynamic flags names.
 * @param[in] n
 *   The number of elements in the names array.
 *
 * @return
 *   The number of dynamic flags that were copied if not negative.
 *   Otherwise:
 *   - ENOMEM - not enough entries in the array
 *   - EINVAL - invalid array entry
 */
__rte_experimental
int rte_pmd_mlx5_get_dyn_flag_names(char *names[], unsigned int n);

#define MLX5_DOMAIN_BIT_NIC_RX	(1 << 0) /**< NIC RX domain bit mask. */
#define MLX5_DOMAIN_BIT_NIC_TX	(1 << 1) /**< NIC TX domain bit mask. */
#define MLX5_DOMAIN_BIT_FDB	(1 << 2) /**< FDB (TX + RX) domain bit mask. */

/**
 * Synchronize the flows to make them take effort on hardware.
 * It only supports DR flows now. For DV and Verbs flows, there is no need to
 * call this function, and a success will return directly in case of Verbs.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[in] domains
 *   Refer to "/usr/include/infiniband/mlx5dv.h".
 *   Bitmask of domains in which the synchronization will be done.
 *   MLX5_DOMAIN_BIT* macros are used to specify the domains.
 *   An ADD or OR operation could be used to synchronize flows in more than
 *   one domain per call.
 *
 * @return
 *   - (0) if successful.
 *   - Negative value if an error.
 */
__rte_experimental
int rte_pmd_mlx5_sync_flow(uint16_t port_id, uint32_t domains);

/**
 * External Rx queue rte_flow index minimal value.
 */
#define MLX5_EXTERNAL_RX_QUEUE_ID_MIN (UINT16_MAX - 1000 + 1)

/**
 * Tag level to set the linear hash index.
 */
#define MLX5_LINEAR_HASH_TAG_INDEX 255

/**
 * Update mapping between rte_flow queue index (16 bits) and HW queue index (32
 * bits) for RxQs which is created outside the PMD.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[in] dpdk_idx
 *   Queue index in rte_flow.
 * @param[in] hw_idx
 *   Queue index in hardware.
 *
 * @return
 *   0 on success, a negative errno value otherwise and rte_errno is set.
 *   Possible values for rte_errno:
 *   - EEXIST - a mapping with the same rte_flow index already exists.
 *   - EINVAL - invalid rte_flow index, out of range.
 *   - ENODEV - there is no Ethernet device for this port id.
 *   - ENOTSUP - the port doesn't support external RxQ.
 */
__rte_experimental
int rte_pmd_mlx5_external_rx_queue_id_map(uint16_t port_id, uint16_t dpdk_idx,
					  uint32_t hw_idx);

/**
 * Remove mapping between rte_flow queue index (16 bits) and HW queue index (32
 * bits) for RxQs which is created outside the PMD.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[in] dpdk_idx
 *   Queue index in rte_flow.
 *
 * @return
 *   0 on success, a negative errno value otherwise and rte_errno is set.
 *   Possible values for rte_errno:
 *   - EINVAL - invalid index, out of range, still referenced or doesn't exist.
 *   - ENODEV - there is no Ethernet device for this port id.
 *   - ENOTSUP - the port doesn't support external RxQ.
 */
__rte_experimental
int rte_pmd_mlx5_external_rx_queue_id_unmap(uint16_t port_id,
					    uint16_t dpdk_idx);

/**
 * Enable traffic for external SQ.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[in] sq_num
 *   SQ HW number.
 *
 * @return
 *   0 on success, a negative errno value otherwise and rte_errno is set.
 *   Possible values for rte_errno:
 *   - EINVAL - invalid sq_number or port type.
 *   - ENODEV - there is no Ethernet device for this port id.
 */
__rte_experimental
int rte_pmd_mlx5_external_sq_enable(uint16_t port_id, uint32_t sq_num);

/**
 * The rate of the host port shaper will be updated directly at the next
 * available descriptor threshold event to the rate that comes with this flag set;
 * set rate 0 to disable this rate update.
 * Unset this flag to update the rate of the host port shaper directly in
 * the API call; use rate 0 to disable the current shaper.
 */
#define MLX5_HOST_SHAPER_FLAG_AVAIL_THRESH_TRIGGERED 0

/**
 * Configure a HW shaper to limit Tx rate for a host port.
 * The configuration will affect all the ethdev ports belonging to
 * the same rte_device.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[in] rate
 *   Unit is 100Mbps, setting the rate to 0 disables the shaper.
 * @param[in] flags
 *   Host shaper flags.
 * @return
 *   0 : operation success.
 *   Otherwise:
 *   - ENOENT - no ibdev interface.
 *   - EBUSY  - the register access unit is busy.
 *   - EIO    - the register access command meets IO error.
 */
__rte_experimental
int rte_pmd_mlx5_host_shaper_config(int port_id, uint8_t rate, uint32_t flags);

struct flexio_process;
struct flexio_outbox;
struct rte_pmd_mlx5_dev_process;
struct rte_pmd_mlx5_dev_ctx;
struct rte_pmd_mlx5_dev_table;
struct rte_pmd_mlx5_dev_action;

enum rte_pmd_mlx5_host_device_type {
	RTE_PMD_MLX5_DEVICE_TYPE_LOCAL,
	RTE_PMD_MLX5_DEVICE_TYPE_DPA,
};

struct rte_pmd_mlx5_host_device_info {
	enum rte_pmd_mlx5_host_device_type type;
	union {
		struct {
			struct flexio_process *process;
			struct flexio_outbox *outbox;
		} dpa;
	};
	/* Number of device queue */
	uint16_t queues;
	/* Max number of queued rule create delete operations  */
	uint16_t queue_size;
};

struct rte_pmd_mlx5_host_action {
	enum rte_flow_action_type type;
	/* Table to bind the actions from */
	struct rte_flow_template_table *table;
	union {
		struct {
			/* ID of destination table */
			uint32_t index;
		} jump;

		struct {
			/* Action template index containing modify fields */
			uint8_t template_index;
		} modify_field;

		struct {
			struct mlx5dv_devx_obj *obj;
			uint32_t id;
		} count;
	};
};

/**
 * IPsec private item
 */
#define RTE_PMD_MLX5_FLOW_ITEM_TYPE_IPSEC_SYNDROME (INT_MIN + 1000000 + 4)
#define RTE_RMD_MLX5_RTE_FLOW_ACTION_TYPE_IPSEC (INT_MIN + 1000000 + 2)

struct mlx5_flow_item_ipsec_syndrome {
	union {
		uint8_t syndrome;
		uintptr_t pad;
	};
};

/**
 * Open new device process, the device can be DPA, ARM or Other.
 * All used  mlx5dr objects should be binded to the device process inorder
 * to be used on the device.
 * NOTE: This function should only be called from the host.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[in] info
 *   The device info.
 * @return device process on success NULL otherwise.
 */
__rte_experimental
struct rte_pmd_mlx5_dev_process *
rte_pmd_mlx5_host_process_open(uint16_t port_id,
			       struct rte_pmd_mlx5_host_device_info *info);

/**
 * Close device process.
 * NOTE: This function should only be called from the host.
 *
 * @param[in] dev_process
 *   Open device process.
 * @return 0 on success non zero otherwise.
 */
__rte_experimental
int
rte_pmd_mlx5_host_process_close(struct rte_pmd_mlx5_dev_process *dev_process);

/**
 * Get mlx5dr context from process used in mlx5dr_dev.
 * NOTE: This function should only be called from the host.
 *
 * @param[in] dev_process
 *   Open device process.
 * @return device context on success NULL otherwise.
 */
__rte_experimental
struct rte_pmd_mlx5_dev_ctx *
rte_pmd_mlx5_host_get_dev_ctx(struct rte_pmd_mlx5_dev_process *dev_process);

/**
 * Bind existing rte table to device process.
 * NOTE: This function should only be called from the host.
 *
 * @param[in] dev_process
 *   Open device process.
 * @param[in] table
 *   Existing rte table.
 * @return device table on success NULL otherwise.
 */
__rte_experimental
struct rte_pmd_mlx5_dev_table *
rte_pmd_mlx5_host_table_bind(struct rte_pmd_mlx5_dev_process *dev_process,
			     struct rte_flow_template_table *table);

/**
 * Unbind rte table from device process.
 * NOTE: This function should only be called from the host.
 *
 * @param[in] dev_process
 *   Open device process.
 * @param[in] dev_table
 *   Binded device table.
 * @return 0 on success non zero otherwise
 */
__rte_experimental
int
rte_pmd_mlx5_host_table_unbind(struct rte_pmd_mlx5_dev_process *dev_process,
			       struct rte_pmd_mlx5_dev_table *dev_table);

/**
 * Bind existing rte action to device process.
 * NOTE: This function should only be called from the host.
 *
 * @param[in] dev_process
 *   Open device process.
 * @param[in] action
 *   rte flow action type to data for binding.
 * @return device action on success NULL otherwise.
 */
__rte_experimental
struct rte_pmd_mlx5_dev_action *
rte_pmd_mlx5_host_action_bind(struct rte_pmd_mlx5_dev_process *dev_process,
			      struct rte_pmd_mlx5_host_action *action);

/**
 * Unbind dev action from device process.
 * NOTE: This function should only be called from the host.
 *
 * @param[in] dev_process
 *   Open device process.
 * @param[in] dev_action
 *   Binded device action.
 * @return 0 on success non zero otherwise.
 */
__rte_experimental
int
rte_pmd_mlx5_host_action_unbind(struct rte_pmd_mlx5_dev_process *dev_process,
				struct rte_pmd_mlx5_dev_action *dev_action);

/**
 * Export Tx UAR created by DPDK for the input port.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[out] uar
 *   UAR object to return
 * @return
 *   0 : operation success.
 *   Otherwise:
 *   - ENODEV - Invalid port_id.
 *   - EINVAL - Invalid argument.
 */
__rte_experimental
int
rte_pmd_mlx5_export_uar(uint16_t port_id, void **uar);

/**
 * Convert UTC timestamp in the corresponding clock queue wci.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @param[in] ts
 *   UTC timestamp
 * @return
 *   Clock queue event index.
 */
__rte_experimental
uint32_t
rte_pmd_mlx5_txpp_convert_tx_ts(uint16_t port_id, uint64_t ts);

/**
 * Return clock queue id created on a device.
 *
 * @param[in] port_id
 *   The port identifier of the Ethernet device.
 * @return
 *   Clock queue id on success.
 *   Otherwise:
 *   - ENODEV - Invalid port_id.
 *   - ENOTSUP - Clock queue not created.
 */
__rte_experimental
int
rte_pmd_mlx5_txpp_idx(uint16_t port_id);

/*
 * IPsec private action configuration
 */
enum rte_pmd_mlx5_ipsec_reformat_type {
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_NONE = 0,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TRANSPORT_IPV4_ENCAP,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TRANSPORT_IPV6_ENCAP,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TRANSPORT_IPV4_UDP_ENCAP,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TRANSPORT_IPV6_UDP_ENCAP,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TUNNEL_ENCAP,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TRANSPORT_DECAP,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TRANSPORT_UDP_DECAP,
	RTE_PMD_MLX5_IPSEC_REFORMAT_TYPE_TUNNEL_DECAP
};

enum rte_pmd_mlx5_ipsec_crypto_type {
	RTE_PMD_MLX5_IPSEC_CRYPTO_TYPE_NONE = 0,
	RTE_PMD_MLX5_IPSEC_CRYPTO_TYPE_ENCRYPT,
	RTE_PMD_MLX5_IPSEC_CRYPTO_TYPE_DECRYPT
};

struct rte_pmd_mlx5_flow_action_ipsec {
	/*
	 * One of NONE/ENCAP/DECAP. This is not the flag bitmask,
	 * because reformat actions are mutual exclusive.
	 */
	enum rte_pmd_mlx5_ipsec_reformat_type reformat_type;
	/*
	 * Reformat header parameters section.
	 */
	size_t reformat_data_sz;
	void *reformat_data;
	/*
	 * Crypto operation NONE/ENCRYPT/DECRYPT
	 */
	enum rte_pmd_mlx5_ipsec_crypto_type crypto_type;
	/*
	 * Crypto operation (IPsec offload) object
	 */
	int ipsec_id;
	/*
	 * Forward action section
	 * RSS, JUMP, PORT, DROP, RSS
	 */
	struct rte_flow_action fwd;
};

#ifdef __cplusplus
}
#endif

#endif /* RTE_PMD_PRIVATE_MLX5_H_ */
